Sets
n nodes /n1*n4/
f fuels /f1*f4/
t time /t1*t2/

$ontext
nodes: 1   2   3     4
       NO  DK  DENE  GB

fuels:   1      2       3     4
         Hydro  Solar   Wind  Thermal
$offtext

alias (n,m,mm);

table A(n,t)         Intercept of inverse demand curve in node n and time t
*  2010   2020
   t1     t2
n1 0.580  0.580
n2 0.580  0.580
n3 0.580  0.580
n4 0.580  0.580
;

table B(n,t)          Slope of inverse demand curve in node n and time t
*  2010   2020
   t1     t2
n1 0.035  0.034
n2 0.124  0.117
n3 0.008  0.009
n4 0.014  0.014
;

*UNIT: EUR/GW
table CG(n,f,t)            Generation cost for fuel f in node n (Equal for all time periods)
*      2010   2020
       t1     t2
n1.f1  0.04   0.047
n1.f2  0      0
n1.f3  0      0
n1.f4  0.043  0.052

n2.f1  0.040  0.047
n2.f2  0      0
n2.f3  0      0
n2.f4  0.043  0.052

n3.f1  0.040  0.047
n3.f2  0      0
n3.f3  0      0
n3.f4  0.043  0.052

n4.f1  0.040  0.047
n4.f2  0      0
n4.f3  0      0
n4.f4  0.043  0.052
;

$ontext
Unit: GW
Fuels: 1       2       3        4
       Hydro    Solar   Wind   Thermal
$offtext

table GCAP(n,f,t)    Generation capacity of fuel f in node n in time t. (Equal for all time periods)
*  Winter Summer Winter Summer
       t1      t2
n1.f1  28.37   42
n1.f2  0.01    0.03
n1.f3  0.520   3
n1.f4  0.237   3.141

n2.f1  0.009   0.009
n2.f2  0.017   0.017
n2.f3  3.952   5
n2.f4  10.775  9

n3.f1  4.821  3.037
n3.f2  25.24  30.146
n3.f3  31.343 70
n3.f4  137.57 127.5

n4.f1  1.676  1.676
n4.f2  0.981  1
n4.f3  6.556  20
n4.f4  81.92  68
;


table AV(n,f,t)  Availability generation of type f in node n at time t.
*      2010   2020
       t1     t2
n1.f1  1      1
n1.f2  0.12   0.12
n1.f3  0.2935 0.2935
n1.f4  1      1
n2.f1  1      1
n2.f2  0.12   0.12
n2.f3  0.2935 0.2935
n2.f4  1      1
n3.f1  1      1
n3.f2  0.12   0.12
n3.f3  0.2935 0.2935
n3.f4  1      1
n4.f1  1      1
n4.f2  0.12   0.12
n4.f3  0.2935 0.2935
n4.f4  1      1
;

*UNIT: GW
table FCAP(n,m,t)      Invested capacity on line between node n and m in time t
*      2010     2020
       t1       t2
n1.n1  200      200
n1.n2  1.000    1.000
n1.n3  0.7      0.7
n1.n4  0        0
n2.n1  1.000    1.000
n2.n2  200      200
n2.n3  2.1      2.1
n2.n4  0        0
n3.n1  0.7      0.7
n3.n2  2.1      2.1
n3.n3  200      200
n3.n4  1.000    1.000
n4.n1  0        0
n4.n2  0        0
n4.n3  1.000    1.000
n4.n4  200      200
;

*UNIT: EUR/GWH
table CL(n,m)            Investment cost if node 1 is LEADER
*  NOR     DK       DENE    UK
   n1      n2       n3      n4
n1 10      0.0017   0.0017  0.0017
n2 10      10       0.0017  0.0017
n3 10      10       10      10
n4 10      10       10      10
;


Variables
w
p_t(n,m,t)
p_el(n,t)
;

Positive Variables
x(n,m)
q_prod(n,f,t)
q_sold(n,m,t)
d(n,t)
flow(n,m,t)
;

equations
eqnWelfare               The objective function
eqnGenCap(n,f,t)         Generatioon capacity
eqnSold(n,t)             Amount sold from node n to m
eqnFlow(n,m,t)           Flow from node n to m
eqnFlowCap(n,m,t)        Transmission capacity on lines
eqnMCC(n,t)
;

eqnWelfare.. sum((n,t), A(n,t)*d(n,t) - 0.5* B(n,t)*d(n,t)*d(n,t)) - sum((n,f,t), CG(n,f,t)*q_prod(n,f,t))
- sum((n,m), CL(n,m)*x(n,m))  =e= w;

eqnGenCap(n,f,t).. q_prod(n,f,t) =l= GCAP(n,f,t)*AV(n,f,t);

eqnSold(n,t).. sum(f, q_prod(n,f,t)) - sum(m, q_sold(n,m,t))  =e= 0;

eqnFlow(n,m,t)..flow(n,m,t) - q_sold(n,m,t) =e= 0;

eqnFlowCap(n,m,t).. flow(n,m,t) + flow(m,n,t)  - FCAP(n,m,t) -x(n,m) -x(m,n)  =l= 0;

eqnMCC(n,t).. d(n,t) - sum(m, q_sold(m,n,t))  =e=  0;


Model it / all  /;

Solve it maximizing w using nlp;
