Sets
n nodes /n1*n4/
f fuels /f1*f4/
t time /t1*t2/

$ontext
nodes: 1   2   3     4
       NO  DK  DENE  GB

fuels:   1      2       3     4
         Hydro  Solar   Wind  Thermal
$offtext

alias (n,m,mm);

table A(n,t)         Intercept of inverse demand curve in node n and time t
*  2010   2020
   t1     t2
n1 0.580  0.580
n2 0.580  0.580
n3 0.580  0.580
n4 0.580  0.580
;

table B(n,t)          Slope of inverse demand curve in node n and time t
*  2010   2020
   t1     t2
n1 0.035  0.034
n2 0.124  0.117
n3 0.008  0.009
n4 0.014  0.014
;

*UNIT: EUR/GW
table CG(n,f,t)            Generation cost for fuel f in node n (Equal for all time periods)
*      2010   2020
       t1     t2
n1.f1  0.04   0.047
n1.f2  0      0
n1.f3  0      0
n1.f4  0.043  0.052

n2.f1  0.040  0.047
n2.f2  0      0
n2.f3  0      0
n2.f4  0.043  0.052

n3.f1  0.040  0.047
n3.f2  0      0
n3.f3  0      0
n3.f4  0.043  0.052

n4.f1  0.040  0.047
n4.f2  0      0
n4.f3  0      0
n4.f4  0.043  0.052
;

$ontext
Unit: GW
Fuels: 1       2       3        4
       Hydro    Solar   Wind   Thermal
$offtext

table GCAP(n,f,t)    Generation capacity of fuel f in node n in time t. (Equal for all time periods)
*  Winter Summer Winter Summer
       t1      t2
n1.f1  28.37   42
n1.f2  0.01    0.03
n1.f3  0.520   3
n1.f4  0.237   3.141

n2.f1  0.009   0.009
n2.f2  0.017   0.017
n2.f3  3.952   5
n2.f4  10.775  9

n3.f1  4.821  3.037
n3.f2  25.24  30.146
n3.f3  31.343 70
n3.f4  137.57 127.5

n4.f1  1.676  1.676
n4.f2  0.981  1
n4.f3  6.556  20
n4.f4  81.92  68
;


table AV(n,f,t)  Availability generation of type f in node n at time t.
*      2010   2020
       t1     t2
n1.f1  1      1
n1.f2  0.12   0.12
n1.f3  0.2935 0.2935
n1.f4  1      1
n2.f1  1      1
n2.f2  0.12   0.12
n2.f3  0.2935 0.2935
n2.f4  1      1
n3.f1  1      1
n3.f2  0.12   0.12
n3.f3  0.2935 0.2935
n3.f4  1      1
n4.f1  1      1
n4.f2  0.12   0.12
n4.f3  0.2935 0.2935
n4.f4  1      1
;

*UNIT: GW
table FCAP(n,m,t)      Invested capacity on line between node n and m in time t
*      2010     2020
       t1       t2
n1.n1  200      200
n1.n2  1.000    1.000
n1.n3  0.7      0.7
n1.n4  0        0
n2.n1  1.000    1.000
n2.n2  200      200
n2.n3  2.1      2.1
n2.n4  0        0
n3.n1  0.7      0.7
n3.n2  2.1      2.1
n3.n3  200      200
n3.n4  1.000    1.000
n4.n1  0        0
n4.n2  0        0
n4.n3  1.000    1.000
n4.n4  200      200
;

*UNIT: EUR/GWH
table CL(n,m)            Investment cost if node 1 is LEADER
*  NOR     DK       DENE    UK
   n1      n2       n3      n4
n1 10      0.0017   0.0017  0.0017
n2 10      10       0.0017  0.0017
n3 10      10       10      10
n4 10      10       10      10
;


Parameter
K1a /0.06/
K1b /70/
K2a /1/
K2b /70/
K3a /1/
K3b /70/
K4a /1/
K4b /70/
K5a /150/
K5b /1/
K6a /10/
K6b /100/
K7a /200/
K7b /0.1/
;

Variables
w1
p_el(n,t)             Dual of demand constraint
p_t(n,m,t)
phi(n,t)
;

Positive Variables
x(n,m)                MWs invested on line between node n to node m in time period t
q_prod(n,f,t)         MWs generated from fuel f at node n in time period t
q_sold(n,m,t)         MWs sold from node n to node m in time period t
d(n,t)                Demand in node n at time t
flow(n,m,t)           Flow on line from node n to node m in time period t
gamma(n,f,t)
epsilon(n,m,t)
;

Binary variables
r1(n,t)
r2(n,f,t)
r3(n,m,t)
r4(n,m,t)
r5(n,f,t)
r6(n,m)
r7(n,m,t)
;

*Fixing variables in order to speed up solution
r6.fx(n,m)$(ord(n) > ord(m)) = 1;
*r6.fx(n,m) = 1;

equations
eqnWelfare1
eqnX(n,m)
eqnObjDemand(n,t)
eqnObjDemand2(n,t)
eqnObjDemand3(n,t)

eqnObjProd(n,f,t)
eqnObjProd2(n,f,t)
eqnObjProd3(n,f,t)

eqnObjSold(n,m,t)
eqnObjSold2(n,m,t)
eqnObjSold3(n,m,t)

eqnObjFlow(n,m,t)
eqnObjFlow2(n,m,t)
eqnObjFlow3(n,m,t)

eqnObjX(n,m)
eqnObjX2(n,m)
eqnObjX3(n,m)

eqnGenCap(n,f,t)
eqnGenCap2(n,f,t)
eqnGenCap3(n,f,t)

eqnSold(n,t)

eqnFlow(n,m,t)

eqnFlowCap(n,m,t)
eqnFlowCap2(n,m,t)
eqnFlowCap3(n,m,t)

eqnMCC(n,t)
;

*Leader's problem: Maximize his own welfare
eqnWelfare1..  w1 =e= sum(t, - 0.5* B('n1',t)*d('n1',t)*d('n1',t))
                  + sum((t,f), CG('n1',f,t)*q_prod('n1',f,t))
                  - sum((m,t), q_sold('n1',m,t)*p_el(m,t))
                  - sum((m,t), flow('n1',m,t) *p_t('n1',m,t))
                  + 0.5*sum((m), CL('n1',m)*x('n1',m));

eqnX(n,m).. x('n1',m) =l= 100;
*Market problem: Maximizing market welfare

*KKT: Objective function derived with respect to d(n)
eqnObjDemand(n,t)..  -A(n,t) + B(n,t)*d(n,t)  + p_el(n,t)  =g= 0 ;
eqnObjDemand2(n,t).. -A(n,t) + B(n,t)*d(n,t)  + p_el(n,t) =l= K1a*r1(n,t);
eqnObjDemand3(n,t)..  d(n,t) - K1b*(1-r1(n,t)) =l= 0 ;

*KKT: Objective function derived with respect to q_prod
eqnObjProd(n,f,t)..  CG(n,f,t) + gamma(n,f,t) + phi(n,t)   =g= 0;
eqnObjProd2(n,f,t).. CG(n,f,t) + gamma(n,f,t) + phi(n,t)   =l= K2a*r2(n,f,t);
eqnObjProd3(n,f,t).. q_prod(n,f,t) - K2b*(1-r2(n,f,t)) =l= 0;

*KKT: Objective function derived with respect to q_sold
eqnObjSold(n,m,t)..  -phi(n,t) - p_t(n,m,t) - p_el(m,t) =g= 0;
eqnObjSold2(n,m,t).. -phi(n,t) - p_t(n,m,t) - p_el(m,t) =l= K3a*r3(n,m,t);
eqnObjSold3(n,m,t).. q_sold(n,m,t) - K3b*(1-r3(n,m,t)) =l= 0;

*KKT: Objective function derived with respect to flow
eqnObjFlow(n,m,t).. epsilon(n,m,t) + epsilon(m,n,t) + p_t(n,m,t) =g= 0;
eqnObjFlow2(n,m,t)..epsilon(n,m,t)+ epsilon(m,n,t) + p_t(n,m,t) =l= K4a*r4(n,m,t);
eqnObjFlow3(n,m,t).. flow(n,m,t) - K4b*(1-r4(n,m,t)) =l= 0;

*KKT: Objective function derived with respect to investements x
eqnObjX(n,m)..  0.5*CL(n,m) - sum(t, epsilon(n,m,t)) -sum(t, epsilon(m,n,t))  =g= 0;
eqnObjX2(n,m).. 0.5*CL(n,m) - sum(t, epsilon(n,m,t)) -sum(t, epsilon(m,n,t)) =l= K6a*r6(n,m);
eqnObjX3(n,m)$(ord(n) < 1).. x(n,m)  =l= K6b*(1-r6(n,m));


*q_prod less than Max Prod
eqnGenCap(n,f,t).. -q_prod(n,f,t) + GCAP(n,f,t)*AV(n,f,t) =g= 0;
eqnGenCap2(n,f,t)..-q_prod(n,f,t) + GCAP(n,f,t)*AV(n,f,t) =l= K5a*r5(n,f,t);
eqnGenCap3(n,f,t)..   gamma(n,f,t) - K5b*(1-r5(n,f,t)) =l= 0;

*sold = prod
eqnSold(n,t)..  sum(f, q_prod(n,f,t))- sum(m, q_sold(n,m,t)) =e= 0;

*flow = sold
eqnFlow(n,m,t).. flow(n,m,t) - q_sold(n,m,t) =e= 0 ;

*Flow less than Max Flow
eqnFlowCap(n,m,t)..  -flow(n,m,t) - flow(m,n,t) +  FCAP(n,m,t) + x(n,m) + x(m,n)  =g= 0 ;
eqnFlowCap2(n,m,t).. -flow(n,m,t) - flow(m,n,t) +  FCAP(n,m,t) + x(n,m) + x(m,n) =l= K7a*r7(n,m,t) ;
eqnFlowCap3(n,m,t)..  epsilon(n,m,t) - K7b*(1-r7(n,m,t)) =l= 0 ;

*Market clearing
eqnMCC(n,t).. d(n,t) - sum(m, q_sold(m,n,t)) =e= 0;

model it /all/;

option reslim = 3600;
option MINLP = BARON;
option optcr = 0.00009;
it.optfile = 1;

solve it using MINLP minimizing w1;